@@ -35,6 +35,7 @@
 #include "alloc.h"
 #include "axis.h"
 #include "misc.h"
+#include "plot2d.h"
 #include "pm3d.h"	/* for is_plot_with_palette */
 
 #define ERRORBARTIC GPMAX((t->h_tic/2),1)
@@ -632,9 +633,14 @@ boundary(struct curve_points *plots, int count)
 	    R_AXIS.max = axis_array[FIRST_X_AXIS].max;
 	    int_warn(NO_CARET, "resetting rrange");
 	}
-	setup_tics(&axis_array[POLAR_AXIS], 10);
-    }
 
+	setup_tics(&axis_array[POLAR_AXIS], 10);
+
+	/* DEBUG: setup_tics may extend rrange to next tic, in which case the
+	 * previous x/y range limits from polar_range_fiddling are out of date.
+	 */
+	polar_range_fiddling(&axis_array[FIRST_X_AXIS], &axis_array[FIRST_Y_AXIS]);
+    }
 
     /* Modify the bounding box to fit the aspect ratio, if any was given */
     if (aspect_ratio != 0.0) {
@@ -822,7 +828,6 @@ boundary(struct curve_points *plots, int count)
     /* Calculate limiting bounds of the key */
     do_key_bounds(key);
 
-
     /* Set default clipping to the plot boundary */
     clip_area = &plot_bounds;
 
@@ -840,10 +845,19 @@ void
 do_key_bounds(legend_key *key)
 {
     struct termentry *t = term;
+    double dx, dy;
 
     key_height = key_title_height + key_title_extra
 		+ key_rows * key_entry_height + key->height_fix * key_entry_height;
-    key_width = key_col_wth * key_cols;
+
+    if (key->user_width.x == 0)
+	key_width = key_col_wth * key_cols;
+    if (key->user_width.x > 0) {
+	if (key->user_width.scalex == screen)
+	    key_width = key->user_width.x * (double)(t->xmax-1);
+	else if (key->user_width.scalex == graph)
+	    key_width = key->user_width.x * (plot_bounds.xright - plot_bounds.xleft);
+    }
 
     /* Key inside plot boundaries */
     if (key->region == GPKEY_AUTO_INTERIOR_LRTBC
@@ -948,6 +962,15 @@ do_key_bounds(legend_key *key)
 	    key->bounds.ytop += key_height;
 	key->bounds.ybot = key->bounds.ytop - key_height;
     }
+
+    /* Regardless of how the key was nominally positioned,
+     * the result can be manually tweaked by "set key offset dx, dy"
+     */
+    map_position_r(&key->offset, &dx, &dy, "key");
+    key->bounds.ytop += dy;
+    key->bounds.ybot += dy;
+    key->bounds.xleft += dx;
+    key->bounds.xright += dx;
 }
 
 /* Calculate positioning of components that make up the key box */
@@ -1017,57 +1040,57 @@ do_key_layout(legend_key *key)
     }
     key_point_offset = (key_sample_left + key_sample_right) / 2;
 
-    /* advance width for cols */
+    /* advance width for cols (initial estimate; may change) */
     key_col_wth = key_size_left + key_size_right;
 
-    key_rows = ptitl_cnt;
-    key_cols = 1;
+    /* New option "require columns <ncol>" */
+    if (key->user_cols > 0) {
+	key_cols = key->user_cols;
+	key_rows = ceil((double)ptitl_cnt / (double)key_cols);
 
-    /* calculate rows and cols for key */
-
-    if (key->stack_dir == GPKEY_HORIZONTAL) {
-	/* maximise no cols, limited by label-length */
-	key_cols = (int) (plot_bounds.xright - plot_bounds.xleft) / key_col_wth;
-	if (key->maxcols > 0 && key_cols > key->maxcols)
-	    key_cols = key->maxcols;
-	/* EAM Dec 2004 - Rather than turn off the key, try to squeeze */
-	if (key_cols == 0) {
-	    key_cols = 1;
-	    key_panic = TRUE;
-	    key_col_wth = (plot_bounds.xright - plot_bounds.xleft);
-	}
-	key_rows = (ptitl_cnt + key_cols - 1) / key_cols;
-	/* now calculate actual no cols depending on no rows */
-	key_cols = (key_rows == 0) ? 1 : (ptitl_cnt + key_rows - 1) / key_rows;
-	if (key_cols == 0) {
-	    key_cols = 1;
-	}
     } else {
-	/* maximise no rows, limited by plot_bounds.ytop-plot_bounds.ybot */
-	int i = (plot_bounds.ytop - plot_bounds.ybot - key->height_fix * key_entry_height
-		    - key_title_height - key_title_extra)
-		/ key_entry_height;
-	if (key->maxrows > 0 && i > key->maxrows)
-	    i = key->maxrows;
+	/* calculate rows and cols for key */
+	key_rows = ptitl_cnt;
+	key_cols = 1;
 
-	if (i == 0) {
-	    i = 1;
-	    key_panic = TRUE;
-	}
-	if (ptitl_cnt > i) {
-	    key_cols = (ptitl_cnt + i - 1) / i;
-	    /* now calculate actual no rows depending on no cols */
+	if (key->stack_dir == GPKEY_HORIZONTAL) {
+	    /* maximise no cols, limited by label-length */
+	    key_cols = (int) (plot_bounds.xright - plot_bounds.xleft) / key_col_wth;
+	    if (key->maxcols > 0 && key_cols > key->maxcols)
+		key_cols = key->maxcols;
+	    /* EAM Dec 2004 - Rather than turn off the key, try to squeeze */
 	    if (key_cols == 0) {
 		key_cols = 1;
 		key_panic = TRUE;
+		key_col_wth = (plot_bounds.xright - plot_bounds.xleft);
 	    }
 	    key_rows = (ptitl_cnt + key_cols - 1) / key_cols;
-#if (0)
+	    /* now calculate actual no cols depending on no rows */
+	    key_cols = (key_rows == 0) ? 1 : (ptitl_cnt + key_rows - 1) / key_rows;
+	    if (key_cols == 0) {
+		key_cols = 1;
+	    }
 	} else {
-	    /* This was a work-around for a spiderplot bug. No longer needed? */
-	    if (key_rows == 0)
-		key_rows = i;
-#endif
+	    /* maximise no rows, limited by plot_bounds.ytop-plot_bounds.ybot */
+	    int i = (plot_bounds.ytop - plot_bounds.ybot - key->height_fix * key_entry_height
+			- key_title_height - key_title_extra)
+		    / key_entry_height;
+	    if (key->maxrows > 0 && i > key->maxrows)
+		i = key->maxrows;
+
+	    if (i == 0) {
+		i = 1;
+		key_panic = TRUE;
+	    }
+	    if (ptitl_cnt > i) {
+		key_cols = (ptitl_cnt + i - 1) / i;
+		/* now calculate actual no rows depending on no cols */
+		if (key_cols == 0) {
+		    key_cols = 1;
+		    key_panic = TRUE;
+		}
+		key_rows = (ptitl_cnt + key_cols - 1) / key_cols;
+	    }
 	}
     }
 
@@ -1077,7 +1100,8 @@ do_key_layout(legend_key *key)
 	if (key->title.font)
 	    t->set_font(key->title.font);
 	ytlen *= t->h_char;
-	if (ytlen > key_cols * key_col_wth)
+	if (ytlen > key_cols * key_col_wth
+	&&  key->user_width.x == 0)
 	    key_col_wth = ytlen / key_cols;
 	if (key->title.font)
 	    t->set_font("");
@@ -1144,6 +1168,8 @@ find_maxl_keys(struct curve_points *plots, int count, int *kcnt)
 	&&  !this_plot->title_position) {
 	    if (this_plot->plot_style == SPIDERPLOT && this_plot->plot_type != KEYENTRY)
 		; /* Nothing */
+	    if (this_plot->plot_style == HISTOGRAMS && this_plot->histogram->type == HT_STACKED_IN_TOWERS)
+		; /* These titles are placed as xtics rather than in the key */
 	    else {
 		ignore_enhanced(this_plot->title_no_enhanced);
 		len = estimate_strlen(this_plot->title, NULL);
@@ -1291,7 +1317,7 @@ do_key_sample(
 	    /* need_fill_border will set the border linetype, but candlesticks don't want it */
 	    if ((this_plot->plot_style == CANDLESTICKS && fs->border_color.type == TC_LT
 							&& fs->border_color.lt == LT_NODRAW)
-	    ||   style == FS_EMPTY
+	    ||   (style == FS_EMPTY && this_plot->plot_style != POLYGONS)
 	    ||   need_fill_border(fs)) {
 		newpath();
 		draw_clip_line( xl + key_sample_left,  yl - key_sample_height/4,
@@ -1375,6 +1401,7 @@ do_key_sample_point(
     struct termentry *t = term;
     int xl_save = xl;
     int yl_save = yl;
+    int interval = this_plot->lp_properties.p_interval;
 
     /* If the plot this title belongs to specified a non-standard place
      * for the key sample to appear, use that to override xl, yl.
@@ -1389,13 +1416,16 @@ do_key_sample_point(
 
     (t->layer)(TERM_LAYER_BEGIN_KEYSAMPLE);
 
-    if (this_plot->plot_style == LINESPOINTS
-	 &&  this_plot->lp_properties.p_interval < 0) {
+    if ((this_plot->plot_style == LINESPOINTS && interval < 0)
+    ||  (this_plot->plot_style == YERRORBARS)) {
 	t_colorspec background_fill = BACKGROUND_COLORSPEC;
-	(*t->set_color)(&background_fill);
-	(*t->pointsize)(pointsize * pointintervalbox);
-	(*t->point)(xl + key_point_offset, yl, 6);
-	term_apply_lp_properties(&this_plot->lp_properties);
+	if ((this_plot->lp_properties.p_type != -1)
+	&&  (pointintervalbox > 0)) {
+	    (*t->set_color)(&background_fill);
+	    (*t->pointsize)(pointsize * pointintervalbox);
+	    (*t->point)(xl + key_point_offset, yl, 6);
+	    term_apply_lp_properties(&this_plot->lp_properties);
+	}
     }
 
     if (this_plot->plot_style == BOXPLOT) {
