@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: contour.c,v 1.29.2.1 2012/08/27 20:24:23 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - contour.c */
 
 /*[
@@ -60,6 +56,8 @@ t_contour_levels_kind contour_levels_kind = LEVELS_AUTO;
 int contour_levels = DEFAULT_CONTOUR_LEVELS;
 int contour_order = DEFAULT_CONTOUR_ORDER;
 int contour_pts = DEFAULT_NUM_APPROX_PTS;
+int contour_firstlinetype = -1;
+TBOOLEAN contour_sortlevels = FALSE;
 
 /* storage for z levels to draw contours at */
 dynarray dyn_contour_levels_list;
@@ -86,7 +84,7 @@ typedef enum en_edge_position {
 
 typedef struct edge_struct {
     struct poly_struct *poly[2]; /* Each edge belongs to up to 2 polygons */
-    struct coordinate GPHUGE *vertex[2]; /* The two extreme points of this edge. */
+    struct coordinate *vertex[2]; /* The two extreme points of this edge. */
     struct edge_struct *next;	/* To chain lists */
     TBOOLEAN is_active;		/* is edge is 'active' at certain Z level? */
     t_edge_position position;	/* position of edge in mesh */
@@ -98,7 +96,7 @@ typedef struct poly_struct {
 } poly_struct;
 
 /* Contours are saved using this struct list. */
-typedef struct cntr_struct {		
+typedef struct cntr_struct {
     double X, Y;		/* The coordinates of this vertex. */
     struct cntr_struct *next;	/* To chain lists. */
 } cntr_struct;
@@ -114,82 +112,91 @@ static t_contour_kind interp_kind = CONTOUR_KIND_LINEAR;
 static double x_min, y_min, z_min;	/* Minimum values of x, y, and z */
 static double x_max, y_max, z_max;	/* Maximum values of x, y, and z */
 
-static void add_cntr_point __PROTO((double x, double y));
-static void end_crnt_cntr __PROTO((void));
-static void gen_contours __PROTO((edge_struct *p_edges, double z_level,
+static void add_cntr_point(double x, double y);
+static void end_crnt_cntr(void);
+static void gen_contours(edge_struct *p_edges, double z_level,
 				  double xx_min, double xx_max,
-				  double yy_min, double yy_max));
-static int update_all_edges __PROTO((edge_struct *p_edges,
-				     double z_level));
-static cntr_struct *gen_one_contour __PROTO((edge_struct *p_edges,
+				  double yy_min, double yy_max);
+static int update_all_edges(edge_struct *p_edges, double z_level);
+static cntr_struct *gen_one_contour(edge_struct *p_edges,
 					     double z_level,
 					     TBOOLEAN *contr_isclosed,
-					     int *num_active));
-static cntr_struct *trace_contour __PROTO((edge_struct *pe_start,
+					     int *num_active);
+static cntr_struct *trace_contour(edge_struct *pe_start,
 					   double z_level,
 					   int *num_active,
-					   TBOOLEAN contr_isclosed));
-static cntr_struct *update_cntr_pt __PROTO((edge_struct *p_edge,
-					    double z_level));
-static int fuzzy_equal __PROTO((cntr_struct *p_cntr1,
-				cntr_struct *p_cntr2));
+					   TBOOLEAN contr_isclosed);
+static cntr_struct *update_cntr_pt(edge_struct *p_edge, double z_level);
+static int fuzzy_equal(cntr_struct *p_cntr1, cntr_struct *p_cntr2);
 
 
-static void gen_triangle __PROTO((int num_isolines,
+static void gen_triangle(int num_isolines,
 				  struct iso_curve *iso_lines,
 				  poly_struct **p_polys,
-				  edge_struct **p_edges));
-static void calc_min_max __PROTO((int num_isolines,
+				  edge_struct **p_edges);
+static void calc_min_max(int num_isolines,
 				  struct iso_curve *iso_lines,
 				  double *xx_min, double *yy_min,
 				  double *zz_min,
 				  double *xx_max, double *yy_max,
-				  double *zz_max));
-static edge_struct *add_edge __PROTO((struct coordinate GPHUGE *point0,
-					     struct coordinate GPHUGE *point1,
+				  double *zz_max);
+static edge_struct *add_edge(struct coordinate *point0,
+					     struct coordinate *point1,
 					     edge_struct
 					     **p_edge,
-					     edge_struct **pe_tail));
-static poly_struct *add_poly __PROTO((edge_struct *edge0,
+					     edge_struct **pe_tail);
+static poly_struct *add_poly(edge_struct *edge0,
 					     edge_struct *edge1,
 					     edge_struct *edge2,
 					     poly_struct **p_poly,
-					     poly_struct **pp_tail));
+					     poly_struct **pp_tail);
 
-static void put_contour __PROTO((cntr_struct *p_cntr,
+static void put_contour(cntr_struct *p_cntr,
 				 double xx_min, double xx_max,
 				 double yy_min, double yy_max,
-				 TBOOLEAN contr_isclosed));
-static void put_contour_nothing __PROTO((cntr_struct *p_cntr));
-static int chk_contour_kind __PROTO((cntr_struct *p_cntr,
-				     TBOOLEAN contr_isclosed));
-static void put_contour_cubic __PROTO((cntr_struct *p_cntr,
+				 TBOOLEAN contr_isclosed);
+static void put_contour_nothing(cntr_struct *p_cntr);
+static int chk_contour_kind(cntr_struct *p_cntr, TBOOLEAN contr_isclosed);
+static void put_contour_cubic(cntr_struct *p_cntr,
 				       double xx_min, double xx_max,
 				       double yy_min, double yy_max,
-				       TBOOLEAN contr_isclosed));
-static void put_contour_bspline __PROTO((cntr_struct *p_cntr,
-					 TBOOLEAN contr_isclosed));
-static void free_contour __PROTO((cntr_struct *p_cntr));
-static int count_contour __PROTO((cntr_struct *p_cntr));
-static int gen_cubic_spline __PROTO((int num_pts, cntr_struct *p_cntr,
+				       TBOOLEAN contr_isclosed);
+static void put_contour_bspline(cntr_struct *p_cntr, TBOOLEAN contr_isclosed);
+static void free_contour(cntr_struct *p_cntr);
+static int count_contour(cntr_struct *p_cntr);
+static int gen_cubic_spline(int num_pts, cntr_struct *p_cntr,
 				     double d2x[], double d2y[],
 				     double delta_t[],
 				     TBOOLEAN contr_isclosed,
-				     double unit_x, double unit_y));
-static void intp_cubic_spline __PROTO((int n, cntr_struct *p_cntr,
+				     double unit_x, double unit_y);
+static void intp_cubic_spline(int n, cntr_struct *p_cntr,
 				       double d2x[], double d2y[],
-				       double delta_t[], int n_intpol));
-static int solve_cubic_1 __PROTO((tri_diag m[], int n));
-static void solve_cubic_2 __PROTO((tri_diag m[], double x[], int n));
-static void gen_bspline_approx __PROTO((cntr_struct *p_cntr,
+				       double delta_t[], int n_intpol);
+static int solve_cubic_1(tri_diag m[], int n);
+static void solve_cubic_2(tri_diag m[], double x[], int n);
+static void gen_bspline_approx(cntr_struct *p_cntr,
 					int num_of_points, int order,
-					TBOOLEAN contr_isclosed));
-static void eval_bspline __PROTO((double t, cntr_struct *p_cntr,
+					TBOOLEAN contr_isclosed);
+static void eval_bspline(double t, cntr_struct *p_cntr,
 				  int num_of_points, int order, int j,
 				  TBOOLEAN contr_isclosed, double *x,
-				  double *y));
-static double fetch_knot __PROTO((TBOOLEAN contr_isclosed, int num_of_points,
-				  int order, int i));
+				  double *y);
+static double fetch_knot(TBOOLEAN contr_isclosed, int num_of_points,
+				  int order, int i);
+
+static int
+reverse_sort(SORTFUNC_ARGS arg1, SORTFUNC_ARGS arg2)
+{
+    double const *p1 = arg2;
+    double const *p2 = arg1;
+
+    if (*p1 > *p2)
+	return 1;
+    if (*p1 < *p2)
+	return -1;
+    return 0;
+}
+
 
 /*
  * Entry routine to this whole set of contouring module.
@@ -199,9 +206,12 @@ contour(int num_isolines, struct iso_curve *iso_lines)
 {
     int i;
     int num_of_z_levels;	/* # Z contour levels. */
+    double *zlist;
     poly_struct *p_polys, *p_poly;
     edge_struct *p_edges, *p_edge;
-    double z = 0, dz = 0;
+    double z = 0;
+    double z0 = 0;
+    double dz = 0;
     struct gnuplot_contours *save_contour_list;
 
     /* HBB FIXME 20050804: The number of contour_levels as set by 'set
@@ -225,6 +235,10 @@ contour(int num_isolines, struct iso_curve *iso_lines)
     crnt_cntr_pt_index = 0;
 
     if (contour_levels_kind == LEVELS_AUTO) {
+	if (nonlinear(&Z_AXIS)) {
+	    z_max = eval_link_function(Z_AXIS.linked_to_primary, z_max);
+	    z_min = eval_link_function(Z_AXIS.linked_to_primary, z_min);
+	}
 	dz = fabs(z_max - z_min);
 	if (dz == 0)
 	    return NULL;	/* empty z range ? */
@@ -232,36 +246,55 @@ contour(int num_isolines, struct iso_curve *iso_lines)
 	 * desired number of contour levels. The "* 2" is historical.
 	 * */
 	dz = quantize_normal_tics(dz, ((int) contour_levels + 1) * 2);
-	z = floor(z_min / dz) * dz;
-	num_of_z_levels = (int) floor((z_max - z) / dz);
+	z0 = floor(z_min / dz) * dz;
+	num_of_z_levels = (int) floor((z_max - z0) / dz);
+	if (num_of_z_levels <= 0)
+	    return NULL;
     }
+
+    /* Build a list of contour levels */
+    zlist = gp_alloc(num_of_z_levels * sizeof(double), NULL);
     for (i = 0; i < num_of_z_levels; i++) {
 	switch (contour_levels_kind) {
 	case LEVELS_AUTO:
-	    z += dz;
+	    z = z0 + (i+1) * dz;
 	    z = CheckZero(z,dz);
+	    if (nonlinear(&Z_AXIS))
+		z = eval_link_function((&Z_AXIS), z);
 	    break;
 	case LEVELS_INCREMENTAL:
-	    z = AXIS_LOG_VALUE(FIRST_Z_AXIS, contour_levels_list[0]) +
-		i * AXIS_LOG_VALUE(FIRST_Z_AXIS, contour_levels_list[1]);
+	    if (Z_AXIS.log)
+		z = contour_levels_list[0] * pow(contour_levels_list[1], (double)i);
+	    else
+		z = contour_levels_list[0] + i * contour_levels_list[1];
 	    break;
 	case LEVELS_DISCRETE:
-	    z = AXIS_LOG_VALUE(FIRST_Z_AXIS, contour_levels_list[i]);
+	    z = contour_levels_list[i];
 	    break;
 	}
+	zlist[i] = z;
+    }
+    /* Sort the list high-to-low if requested */
+    if (contour_sortlevels)
+	qsort(zlist, num_of_z_levels, sizeof(double), reverse_sort);
+
+    /* Create contour line for each z value in the list */
+    for (i = 0; i < num_of_z_levels; i++) {
+	z = zlist[i];
 	contour_level = z;
 	save_contour_list = contour_list;
 	gen_contours(p_edges, z, x_min, x_max, y_min, y_max);
 	if (contour_list != save_contour_list) {
 	    contour_list->isNewLevel = 1;
 	    /* Nov-2011 Use gprintf rather than sprintf so that LC_NUMERIC is used */
-	    gprintf(contour_list->label, sizeof(contour_list->label), 
-		    contour_format, 1.0, AXIS_DE_LOG_VALUE(FIRST_Z_AXIS,z));
+	    gprintf(contour_list->label, sizeof(contour_list->label),
+		    contour_format, 1.0, z);
 	    contour_list->z = z;
 	}
     }
 
     /* Free all contouring related temporary data. */
+    free(zlist);
     while (p_polys) {
 	p_poly = p_polys->next;
 	free(p_polys);
@@ -315,6 +348,7 @@ end_crnt_cntr()
 	cntr->coords[i].z = contour_level;
     }
     cntr->num_pts = crnt_cntr_pt_index;
+    cntr->label[0] = '\0';
 
     cntr->next = contour_list;
     contour_list = cntr;
@@ -529,17 +563,38 @@ update_cntr_pt(edge_struct *p_edge, double z_level)
 }
 
 /* Simple routine to decide if two contour points are equal by
- * calculating the relative error (< EPSILON).  */
-/* HBB 20010121: don't use absolute value 'zero' to compare to data
- * values. */
+ * calculating the relative error (< EPSILON).
+ * Back-port fix for nonlinear axes from version 6.
+ */
 static int
 fuzzy_equal(cntr_struct *p_cntr1, cntr_struct *p_cntr2)
 {
-    double unit_x, unit_y;
-    unit_x = fabs(x_max - x_min);		/* reference */
-    unit_y = fabs(y_max - y_min);
-    return ((fabs(p_cntr1->X - p_cntr2->X) < unit_x * EPSILON)
-	    && (fabs(p_cntr1->Y - p_cntr2->Y) < unit_y * EPSILON));
+    double unit_x = fabs(x_max - x_min);
+    double unit_y = fabs(y_max - y_min);
+    double x1 = p_cntr1->X;
+    double x2 = p_cntr2->X;
+    double y1 = p_cntr1->Y;
+    double y2 = p_cntr2->Y;
+
+    if (nonlinear(&X_AXIS)) {
+	x1 = eval_link_function(X_AXIS.linked_to_primary, x1);
+	x2 = eval_link_function(X_AXIS.linked_to_primary, x2);
+    } else {
+	x1 /= unit_x;
+	x2 /= unit_x;
+    }
+    if (nonlinear(&Y_AXIS)) {
+	y1 = eval_link_function(Y_AXIS.linked_to_primary, y1);
+	y2 = eval_link_function(Y_AXIS.linked_to_primary, y2);
+    } else {
+	y1 /= unit_y;
+	y2 /= unit_y;
+    }
+
+    if (fabs(x1-x2) < EPSILON  &&  fabs(y1-y2) < EPSILON)
+	return TRUE;
+    else
+	return FALSE;
 }
 
 /*
@@ -554,11 +609,9 @@ gen_triangle(
     edge_struct **p_edges)	/* list of edges output */
 {
     int i, j, grid_x_max = iso_lines->p_count;
-    edge_struct *p_edge1, *p_edge2, *edge0, *edge1, *edge2, *pe_tail,
-	   *pe_tail2, *pe_temp;
-    poly_struct *pp_tail, *lower_tri, *upper_tri;
-    /* HBB 980308: need to tag *each* of them as GPHUGE! */
-    struct coordinate GPHUGE *p_vrtx1, GPHUGE * p_vrtx2;
+    edge_struct *p_edge1, *p_edge2, *edge0, *edge1, *edge2, *pe_tail, *pe_tail2, *pe_temp;
+    poly_struct *pp_tail;
+    struct coordinate *p_vrtx1, * p_vrtx2;
 
     (*p_polys) = pp_tail = NULL;	/* clear lists */
     (*p_edges) = pe_tail = NULL;
@@ -642,7 +695,7 @@ gen_triangle(
 		edge1->position = DIAGONAL;
 
 	    /* generate lower triangle */
-	    lower_tri = add_poly(edge0, edge1, edge2, p_polys, &pp_tail);
+	    add_poly(edge0, edge1, edge2, p_polys, &pp_tail);
 
 	    /* copy diagonal edge for upper triangle */
 	    edge0 = edge1;
@@ -654,7 +707,7 @@ gen_triangle(
 	    edge2 = add_edge(p_vrtx1 + j + 1, p_vrtx2 + j + 1, p_edges, &pe_tail);
 
 	    /* generate upper triangle */
-	    upper_tri = add_poly(edge0, edge1, edge2, p_polys, &pp_tail);
+	    add_poly(edge0, edge1, edge2, p_polys, &pp_tail);
 	}
 
 	if (p_edge2) {
@@ -698,7 +751,7 @@ calc_min_max(
     double *xx_max, double *yy_max, double *zz_max) /* min/max values in/out */
 {
     int i, j, grid_x_max;
-    struct coordinate GPHUGE *vertex;
+    struct coordinate *vertex;
 
     grid_x_max = iso_lines->p_count;	/* number of vertices per iso_line */
 
@@ -743,8 +796,8 @@ calc_min_max(
  */
 static edge_struct *
 add_edge(
-    struct coordinate GPHUGE *point0,	/* 2 vertices input */
-    struct coordinate GPHUGE *point1,
+    struct coordinate *point0,	/* 2 vertices input */
+    struct coordinate *point1,
     edge_struct **p_edge, /* pointers to edge list in/out */
     edge_struct **pe_tail)
 {
@@ -1221,7 +1274,7 @@ intp_cubic_spline(
  *   m[i][0] = M_{i,i-1}  for i=1,2,...,n-1    and    m[0][0] = M_{0,n-1} ,
  *   m[i][1] = M_{i, i }  for i=0,1,...,n-1
  *   m[i][2] = M_{i,i+1}  for i=0,1,...,n-2    and    m[n-1][2] = M_{n-1,0}.
- * M should be symmetric (m[i+1][0]=m[i][2]) and positiv definite.
+ * M should be symmetric (m[i+1][0]=m[i][2]) and positive definite.
  * The size of the system is given in n (n>=1).
  *
  * In the first procedure the Cholesky decomposition M = C^T * D * C
@@ -1239,7 +1292,7 @@ solve_cubic_1(tri_diag m[], int n)
 
     d = m[0][1];		/* D_{0,0} = M_{0,0} */
     if (d <= 0.)
-	return FALSE;		/* M (or D) should be positiv definite */
+	return FALSE;		/* M (or D) should be positive definite */
     m_n = m[0][0];		/*  M_{0,n-1}  */
     m_nn = m[n - 1][1];		/* M_{n-1,n-1} */
     for (i = 0; i < n - 2; i++) {
@@ -1250,7 +1303,7 @@ solve_cubic_1(tri_diag m[], int n)
 	m_n = -m[i][2] * m_n;	/* to get C_{i+1,n-1} */
 	d = m[i + 1][1] - m[i][2] * m_ij;	/* D_{i+1,i+1} */
 	if (d <= 0.)
-	    return FALSE;	/* Elements of D should be positiv */
+	    return FALSE;	/* Elements of D should be positive */
 	m[i + 1][1] = d;
     }
     if (n >= 2) {		/* Complete last column */
@@ -1320,7 +1373,7 @@ solve_tri_diag(tri_diag m[], double r[], double x[], int n)
 	m[i][1] = m[i][1] - m[i - 1][2] * t;
 	r[i] = r[i] - r[i - 1] * t;
     }
-    /* Now do back subtitution - update the solution vector X: */
+    /* Now do back substitution - update the solution vector X: */
     if (m[n - 1][1] == 0)
 	return FALSE;
     x[n - 1] = r[n - 1] / m[n - 1][1];	/* Find last element. */
@@ -1456,13 +1509,13 @@ eval_bspline(
  * might be float (Knot(i) = i) or open (where the first and last "order"
  * knots are equal). contr_isclosed determines knot kind - open contour means
  * open knot vector, and closed contour selects float knot vector.
- * Note the knot vector is not exist and this routine simulates it existance
+ * Note the knot vector is not exist and this routine simulates it existence
  * Also note the indexes for the knot vector starts from 0.
  */
 static double
 fetch_knot(TBOOLEAN contr_isclosed, int num_of_points, int order, int i)
 {
-    if(! contr_isclosed) {
+    if (!contr_isclosed) {
 	if (i <= order)
 	    return 0.0;
 	else if (i <= num_of_points)
