@@ -1198,7 +1198,6 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 	POINT corners[4];			/* image corners */
 	int color_mode = 0;			/* image color mode */
 
-#ifdef EAM_BOXED_TEXT
 	struct s_boxedtext {
 		TBOOLEAN boxing;
 		t_textbox_options option;
@@ -1207,7 +1206,6 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 		RECT  box;
 		int   angle;
 	} boxedtext;
-#endif
 
 	/* point symbols */
 	enum win_pointtypes last_symbol = W_invalid_pointtype;
@@ -1293,9 +1291,7 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 		lpgw->keyboxes[i].top = 0;
 	}
 
-#ifdef EAM_BOXED_TEXT
 	boxedtext.boxing = FALSE;
-#endif
 
 	/* do the drawing */
 	blkptr = lpgw->gwopblk_head;
@@ -1473,6 +1469,7 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 
 		case W_polyline: {
 			POINTL * poly = reinterpret_cast<POINTL *>(LocalLock(curptr->htext));
+			if (poly == NULL) break; // memory allocation failed
 			polyi = curptr->x;
 			// FIXME: Test for zero-length segments
 			if (polyi == 2 && poly[0].x == poly[1].x && poly[0].y == poly[1].y) {
@@ -1569,6 +1566,7 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 				hr = d2dCreateStrokeStyle(static_cast<D2D1_DASH_STYLE>(cur_penstruct.lopnStyle), lpgw->rounded, &pStrokeStyle);
 			} else if (dt == DASHTYPE_CUSTOM) {
 				t_dashtype * dash = reinterpret_cast<t_dashtype *>(LocalLock(curptr->htext));
+				if (dash == NULL) break;
 				INT count = 0;
 				while ((dash->pattern[count] != 0.) && (count < DASHPATTERN_LENGTH)) count++;
 				hr = d2dCreateStrokeStyle(dash->pattern, count, lpgw->rounded, &pStrokeStyle);
@@ -1602,11 +1600,7 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 					D2D1_SIZE_F size;
 					int dxl, dxr;
 
-#ifndef EAM_BOXED_TEXT
-					if (keysample) {
-#else
 					if (keysample || boxedtext.boxing) {
-#endif
 						d2dMeasureText(pRenderTarget, textw, pWriteTextFormat, &size);
 						size.height = lpgw->tmHeight;
 						if (lpgw->justify == LEFT) {
@@ -1623,7 +1617,6 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 						d2d_update_keybox(lpgw, pRenderTarget, plotno, xdash - dxl, ydash - size.height / 2);
 						d2d_update_keybox(lpgw, pRenderTarget, plotno, xdash + dxr, ydash + size.height / 2);
 					}
-#ifdef EAM_BOXED_TEXT
 					if (boxedtext.boxing) {
 						if (boxedtext.box.left > (xdash - boxedtext.start.x - dxl))
 							boxedtext.box.left = xdash - boxedtext.start.x - dxl;
@@ -1636,7 +1629,6 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 						/* We have to remember the text angle as well. */
 						boxedtext.angle = lpgw->angle;
 					}
-#endif
 					free(textw);
 				}
 			}
@@ -1656,7 +1648,6 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 					d2d_update_keybox(lpgw, pRenderTarget, plotno, xdash - extend.left, ydash - extend.top);
 					d2d_update_keybox(lpgw, pRenderTarget, plotno, xdash + extend.right, ydash + extend.bottom);
 				}
-#ifdef EAM_BOXED_TEXT
 				if (boxedtext.boxing) {
 					if (boxedtext.box.left > (boxedtext.start.x - xdash - extend.left))
 						boxedtext.box.left = boxedtext.start.x - xdash - extend.left;
@@ -1669,7 +1660,6 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 					/* We have to store the text angle as well. */
 					boxedtext.angle = lpgw->angle;
 				}
-#endif
 			}
 			LocalUnlock(curptr->htext);
 			break;
@@ -1686,7 +1676,6 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 			}
 			break;
 
-#ifdef EAM_BOXED_TEXT
 		case W_boxedtext:
 			if (seq == 0) {
 				boxedtext.option = (t_textbox_options) curptr->x;
@@ -1798,7 +1787,6 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 				break;
 			}
 			break;
-#endif
 
 		case W_fillstyle: {
 			// FIXME: resetting polyi here should not be necessary
@@ -2106,6 +2094,7 @@ d2d_do_draw(LPGW lpgw, ID2D1RenderTarget * pRenderTarget, LPRECT rect, bool inte
 			} else {
 				/* The last OP contains the image and it's size */
 				BYTE * image = reinterpret_cast<BYTE *>(LocalLock(curptr->htext));
+				if (image == NULL) break; // memory allocation failed
 				UINT32 width = curptr->x;
 				UINT32 height = curptr->y;
 				D2D1_SIZE_U size = D2D1::SizeU(width, height);
