@@ -1,7 +1,3 @@
-/*
- * $Id: wgnuplib.h,v 1.89.2.3 2017/06/21 08:43:59 markisch Exp $
- */
-
 /* GNUPLOT - win/wgnuplib.h */
 
 /*[
@@ -62,7 +58,7 @@ extern "C" {
 /* #define DEFAULT_CHARSET ANSI_CHARSET */
 # define MoveTo(hdc,x,y) MoveToEx(hdc,x,y,(LPPOINT)NULL);
 
-/* printf format for TCHAR arguments */ 
+/* printf format for TCHAR arguments */
 #ifdef UNICODE
 # define TCHARFMT "%ls"
 #else
@@ -274,14 +270,17 @@ struct GWOPBLK {			/* kept in local memory */
 	UINT used;				/* number of GWOP's used */
 };
 
+/* Maximum number of GWOPBLK arrays to be remembered. */
+#define GWOPMAX (4*4096)
+
 /* point types */
 enum win_pointtypes {
 	W_invalid_pointtype = 0,
 	W_dot = 10,
-	W_plus, W_cross, W_star, 
+	W_plus, W_cross, W_star,
 	W_box, W_fbox,
-	W_circle, W_fcircle, 
-	W_itriangle, W_fitriangle, 
+	W_circle, W_fcircle,
+	W_itriangle, W_fitriangle,
 	W_triangle, W_ftriangle,
 	W_diamond, W_fdiamond,
 	W_pentagon, W_fpentagon,
@@ -293,7 +292,7 @@ enum win_pointtypes {
 /* ops */
 enum win_draw_commands {
 	W_endoflist = 0,
-	W_point = 9, 
+	W_point = 9,
 	W_pointsize = 30,
 	W_setcolor,
 	W_polyline, W_line_type, W_dash_type, W_line_width,
@@ -310,95 +309,128 @@ enum win_draw_commands {
 
 typedef struct tagGW {
 	GP_LPPRINT	lpr;	/* must be first */
+
+	/* window properties etc. */
 	HINSTANCE hInstance;	/* required */
 	HINSTANCE hPrevInstance;/* required */
 	int	Id;		/* plot number */
 	LPTSTR	Title;		/* required */
-	int	xmax;		/* required */
-	int	ymax;		/* required */
-	LPTW	lptw;		/* optional */  /* associated text window */
-	BOOL	bDocked;	/* is the graph docked to the text window? */
-	POINT	Origin;		/* optional */	/* origin of graph window */
-	POINT	Size;		/* optional */	/* size of graph window */
+	LPTW	lptw;		/* associated text window, optional */
 	LPTSTR	IniFile;	/* optional */
 	LPTSTR	IniSection;	/* optional */
-	HWND	hWndGraph;	/* window handle */
-	HWND	hStatusbar;	/* window handle of status bar */
-	int	StatusHeight;	/* height of status line area */
-	HWND	hToolbar;
-	int	ToolbarHeight;
-	HMENU	hPopMenu;	/* popup menu */
-	HBITMAP	hBitmap;	/* bitmap of current graph */
-	BOOL	buffervalid;	/* indicates if hBitmap is valid */
-	BOOL	rotating;	/* are we currently rotating the graph? */
+
+	/* window size and position */
+	BOOL	bDocked;	/* is the graph docked to the text window? */
+	POINT	Origin;		/* origin of graph window */
+	POINT	Size;		/* size of graph window */
 	POINT	Canvas;		/* requested size of the canvas */
 	POINT	Decoration;	/* extent of the window decorations */
+	int	StatusHeight;	/* height of status line area */
+	int	ToolbarHeight;	/* height of the toolbar */
 
-	struct GWOPBLK *gwopblk_head;
+	/* (subwindow) handles */
+	HWND	hWndGraph;	/* window handle of the top window */
+	HWND	hGraph;		/* window handle of the actual graph */
+	HWND	hStatusbar;	/* window handle of status bar */
+	HWND	hToolbar;	/* window handle of the toolbar */
+	HMENU	hPopMenu;	/* popup menu */
+	HWND	hTooltip;	/* tooltip windows for hypertext */
+
+	/* command list */
+	struct GWOPBLK *gwopblk_head;	/* graph command list */
 	struct GWOPBLK *gwopblk_tail;
 	unsigned int nGWOP;
 	BOOL	locked;		/* locked if being written */
 
+	/* off-screen bitmap used by GDI, GDI+ and D2D DCRenderer */
+	HBITMAP	hBitmap;	/* bitmap of current graph */
+	BOOL	buffervalid;	/* indicates if hBitmap is valid */
+
+	/* state */
 	BOOL	initialized;	/* already initialized? */
+	BOOL	rotating;	/* are we currently rotating the graph? */
+
+	/* options */
 	BOOL	graphtotop;	/* bring graph window to top after every plot? */
-	BOOL	color;		/* color pens? */
-	BOOL	dashed;		/* dashed lines? */
-	BOOL	rounded;	/* rounded line caps and joins? */
+	BOOL	color;		/* colored graph? */
 	BOOL	oversample;	/* oversampling? */
 	BOOL	gdiplus;	/* Use GDI+ only backend? */
-	BOOL	d2d;
+	BOOL	d2d;	/* Use Direct2D backend? */
 	BOOL	antialiasing;	/* anti-aliasing? */
 	BOOL	polyaa;		/* anti-aliasing for polygons ? */
 	BOOL	fastrotation;	/* rotate without anti-aliasing? */
+	COLORREF background;	/* background color */
 
-	BOOL	*hideplot;
-	unsigned int maxhideplots;
-	BOOL	hidegrid;
+	/* plot properties */
+	int	xmax;		/* required */
+	int	ymax;		/* required */
+	int	htic;		/* horizontal size of point symbol (xmax units) */
+	int	vtic;		/* vertical size of point symbol (ymax units)*/
+	int	hchar;		/* horizontal size of character (xmax units) */
+	int	vchar;		/* vertical size of character (ymax units)*/
+
+	/* layers */
 	unsigned int numplots;
 	BOOL	hasgrid;
+	BOOL	hidegrid;
+	BOOL	*hideplot;	/* array for handling hidden plots */
+	unsigned int maxhideplots;
 	LPRECT	keyboxes;
 	unsigned int maxkeyboxes;
 
-	HWND	hTooltip;	/* tooltip windows for hypertext */
+	/* hypertext */
 	struct tooltips * tooltips;
 	unsigned int maxtooltips;
 	unsigned int numtooltips;
 
-	int	htic;		/* horizontal size of point symbol (xmax units) */
-	int 	vtic;		/* vertical size of point symbol (ymax units)*/
-	int	hchar;		/* horizontal size of character (xmax units) */
-	int	vchar;		/* vertical size of character (ymax units)*/
+	/* points and lines */
+	double	pointscale;	/* scale factor for point sizes */
+	double	org_pointsize;	/* original pointsize */
+	BOOL	dashed;		/* dashed lines? */
+	BOOL	rounded;	/* rounded line caps and joins? */
+	double	linewidth;	/* scale factor for linewidth */
+	LOGPEN	colorpen[WGNUMPENS+2];	/* color pen definitions */
+	LOGPEN	monopen[WGNUMPENS+2];	/* mono pen definitions */
 
-	TCHAR	fontname[MAXFONTNAME];	/* current font name */
-	int	fontsize;	/* current font size in pts */
+	/* fonts */
+	double	fontscale;	/* scale factor for font sizes */
 	TCHAR	deffontname[MAXFONTNAME]; /* default font name */
 	int	deffontsize;	/* default font size */
+	TCHAR	fontname[MAXFONTNAME];	/* current font name */
+	int	fontsize;	/* current font size in pts */
 	int	angle;		/* text angle */
-	BOOL	rotate;		/* can text be rotated 90 degrees ? */
+	BOOL	rotate;		/* can text be rotated? */
 	int	justify;	/* text justification */
-	HFONT	hfonth;		/* horizonal font */
-	HFONT	hfontv;		/* rotated font (arbitrary angle) */
-	LOGFONT	lf;			/* cached to speed up rotated fonts */
-	double	org_pointsize;	/* Original Pointsize */
 	int	encoding_error; /* last unknown encoding */
-	double	fontscale;	/* scale factor for font sizes */
-	double	pointscale;	/* scale factor for point sizes */
 	enum set_encoding_id encoding;	/* text encoding */
 	LONG	tmHeight;	/* text metric of current font */
 	LONG	tmAscent;
 	LONG	tmDescent;
 
+#ifdef USE_WINGDI
+	/* GDI resources */
 	HPEN	hapen;		/* stored current pen */
 	HPEN	hsolid;		/* solid sibling of current pen */
 	HPEN	hnull;		/* empty null pen */
-	LOGPEN	colorpen[WGNUMPENS+2];	/* color pen definitions */
-	LOGPEN	monopen[WGNUMPENS+2];	/* mono pen definitions */
-	double	linewidth;	/* scale factor for linewidth */
-
 	HBRUSH	colorbrush[WGNUMPENS+2];   /* brushes to fill points */
-	COLORREF background;	/* background color */
 	HBRUSH	hbrush;		/* background brush */
-	HBRUSH	hcolorbrush;	/* */
+	HBRUSH	hcolorbrush;	/* color fill brush */
+	HFONT	hfonth;		/* horizontal font */
+	HFONT	hfontv;		/* rotated font (arbitrary angle) */
+	LOGFONT	lf;			/* cached to speed up rotated fonts */
+#endif
+
+#ifdef HAVE_D2D
+	/* Direct2D resources */
+#if !defined(HAVE_D2D11) || defined(DCRENDERER)
+	struct ID2D1RenderTarget * pRenderTarget;
+#else
+	struct ID2D1Device * pDirect2dDevice;
+	struct ID2D1DeviceContext * pRenderTarget;
+	struct IDXGISwapChain1 * pDXGISwapChain;
+#endif
+	int		dpi;			/* (nominal) resolution of output device */
+#endif
 
 	struct tagGW * next;	/* pointer to next window */
 } GW;
@@ -423,10 +455,10 @@ typedef struct {
 	int totaldesc;       /* total height below center line */
 	double res_scale;    /* scaling due to different resolution (printers) */
 	int shift;           /* baseline alignment */
-	void (* set_font)();
+	void (* set_font)(void);
 	unsigned (* text_length)(char *);
 	void (* put_text)(int , int, char *);
-	void (* cleanup)();
+	void (* cleanup)(void);
 } enhstate_struct;
 extern enhstate_struct enhstate;
 
@@ -474,8 +506,9 @@ void Graph_set_clipboard(LPGW lpgw, LPCSTR s);
 void GraphEnhancedOpen(char *fontname, double fontsize, double base,
     TBOOLEAN widthflag, TBOOLEAN showflag, int overprint);
 void GraphEnhancedFlush(void);
+LPWSTR UnicodeTextWithEscapes(LPCSTR str, enum set_encoding_id encoding);
 
-void WIN_update_options __PROTO((void));
+void WIN_update_options(void);
 
 
 /* ================================== */
